/**
 * 	Copyright (C) 2011 Sam Macbeth <sm1106 [at] imperial [dot] ac [dot] uk>
 *
 * 	This file is part of Presage2.
 *
 *     Presage2 is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Lesser Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     Presage2 is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU Lesser Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser Public License
 *     along with Presage2.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.ac.imperial.presage2.core.environment;

import java.util.UUID;

import uk.ac.imperial.presage2.core.Action;
import uk.ac.imperial.presage2.core.messaging.Input;

/**
 * 
 * <p>An ActionHandler processes an {@link Action} in the environment.</p>
 * 
 * @author Sam Macbeth
 *
 */
public interface ActionHandler {

	/**
	 * Check if this Handler can handle the given action.
	 * @param action Action to handle
	 * @return true if it can, false otherwise
	 */
	public  boolean canHandle(Action action);

	/**
	 * Handle the given action.
	 * @param action action to handle
	 * @param actor actor performing this action
	 * @return May return a further Input to be returned to the actor, or null if not.
	 */
	public Input handle(Action action, UUID actor) throws ActionHandlingException;
	
}
